unit DD43U1;
(*A program to illustrate some points of Delphi programming.
  There are areas where severe shortcuts have been taken, e.g. the
  lack of flexibility over where the output files go, and their
  names.

  The program is not meant to be "finished", or "good"... but it does
  work, and parts of it are well made. The rest are sketched in in a
  manner which does not obstruct eventual fine tuning.

  The program asks the user for a set of letters and then generates
  all possible combinations of those letters, sorting the resulting
  "words" into separate files, depending on the likelihood of that
  string of letters is used as a word in English.

  Inspired by a wish to pull a friend's leg over the assertion in
  Dan Brown's "Da Vinci Code" that there are 92 words in "planets".
  With a version of this program, I found all of them... but my list
  included about 11,000 others, too!*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const ver='15 Apr 04';
  (*Started 14 Apr 04 after trip to NYC w/EYC.
    To amuse EB, following ref to "planets: 92 words" in
    Da Vinci Code*)

  (*qOther
  MaxLets determines how many letters are allowed in the word
  you are seeing permutations from. It is always best to put
  such things in constants or variables. Use the editor's
  search to see where maxLets appears in the sourcecode. It
  the value were hardcoded, then changing the number of letters
  allowed would not only be a hassle, but also error prone.*)
  maxLets=7;//Max length of start word

type
  TDD43f1 = class(TForm)
    buGo: TButton;
    eStartWord: TEdit;
    buQuit: TButton;
    me1: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure buQuitClick(Sender: TObject);
    procedure eStartWordChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure buGoClick(Sender: TObject);
  private
    { Private declarations }
    (*qOther
    dfOut0, etc, are the file handles for the vaious files to which
    discovered permutations will be written.*)
    dfOut0,dfOut1,dfOut2,dfOut3,dfOut4,dfOut5:text;
    procedure LogResult(sWord:string);
  public
    { Public declarations }
  end;

var
  DD43f1: TDD43f1;

implementation

{$R *.DFM}
{$R+}

procedure TDD43f1.buQuitClick(Sender: TObject);
begin
(*QOther
These "closefile"s probably should be enabled..... but if the
"closefile"s in buGoClick have been executed, these cause
problems.... and in any case, the buQuitClick button will not
terminate the loops just in from of buGoClick's "closefile"s.
You have to incorporate fancier loops to leave the program
susceptable to premature termination.... even an
application.processmessages near the innermost loop won't
fix the problem, though you might think it should! You should
have one there anyway.*)
{closefile(dfOut0);
closefile(dfOut1);
closefile(dfOut2);
closefile(dfOut3);
closefile(dfOut4);
closefile(dfOut5);}
application.terminate;
end;

procedure TDD43f1.LogResult(sWord:string);
(*qOther
The name "LogResult" is perhaps too modest. As soon as other parts
of the program have generated a new permutation, it is passed to
this procedure. Within the procedure, the string is passed to the
function LikelyValid, which returns a number from 0 (Is certainly a
word) to 5 (unlikely to be a word.) The actual coding of LikelyValid
is hardly rigourous.... remember the program was written as a
bit of fun. See if you can improve the program's "intelligence".

Not only does LogResult have a local function, LikelyValid, but
LielyValid itself contains a local function of it's own,
IsVowel, which checks to see if a character is a vowel, and
returns true or false, as the case may be.

While words may be entered in any case, the program converts them
to lower case, and generates permutations of those letters only,
so is you were being meticulous, you would be offened by, say,
"Berlin" coming out as "berlin". Sorry!*)

function LikelyValid(sTmp:string):byte;
(*Returns 0 for words known to be real, 5 for unlikely, and in between.
N.B. Words of 1, 2 or 3 letters are checked against hard-coded
dictionaries. If the word is not in the dictionary, it will be placed
in DD43R5.txt.*)
var bLen:byte;

function IsVowel(chTmp:char):boolean;
begin
isVowel:=false;
if (pos(chTmp,'aeiouy')>0) then IsVowel:=true;
end;

begin (*LikelyValid*)
(*qOther
This code is rather long winded, but not particularly complex, and
all it does is "rate" the likelyhood of a given string of characters
(in sTmp) being an English word. In general, but not without
exception, every candidate starts rated 5 (UNlikely to be English),
and then is promoted to higher and higher rankings if favorable
indications are detected. The ranking system is just a bit of
fun, not extensively researched!*)
bLen:=length(sTmp);
LikelyValid:=5;
if (pos('a',sTmp)>0) or (pos('e',sTmp)>0) or
   (pos('i',sTmp)>0) or (pos('o',sTmp)>0) or
   (pos('u',sTmp)>0) or (pos('y',sTmp)>0)
   then begin (*has a vowel*)
    LikelyValid:=4;

    if (pos('th',sTmp)>0) then LikelyValid:=3;
    if (pos('in',sTmp)>0) then LikelyValid:=3;
    if (pos('he',sTmp)>0) then LikelyValid:=3;
    if (pos('er',sTmp)>0) then LikelyValid:=3;
    if (pos('re',sTmp)>0) then LikelyValid:=3;
    if (pos('on',sTmp)>0) then LikelyValid:=3;
    if (pos('es',sTmp)>0) then LikelyValid:=3;

    if (sTmp[1]='c')          and IsVowel(sTmp[2]) and (bLen>1) then LikelyValid:=2;
    if (copy(sTmp,1,2)='ch')  and IsVowel(sTmp[3]) and (bLen>2) then LikelyValid:=1;
    if (sTmp[1]='p')          and IsVowel(sTmp[2]) and (bLen>1) then LikelyValid:=2;
    if (copy(sTmp,1,2)='pl')  and IsVowel(sTmp[3]) and (bLen>2) then LikelyValid:=1;
    if (sTmp[1]='s')          and IsVowel(sTmp[2]) and (bLen>1) then LikelyValid:=2;
    if (copy(sTmp,1,2)='sh')  and IsVowel(sTmp[3]) and (bLen>2) then LikelyValid:=1;
    if (copy(sTmp,1,2)='st')  and IsVowel(sTmp[3]) and (bLen>2) then LikelyValid:=1;
    if (copy(sTmp,1,3)='str') and IsVowel(sTmp[4]) and (bLen>3) then LikelyValid:=1;
    if (copy(sTmp,1,2)='th')  and IsVowel(sTmp[3]) and (bLen>2) then LikelyValid:=1;
    if (copy(sTmp,1,3)='thr') and IsVowel(sTmp[4]) and (bLen>3) then LikelyValid:=1;

    if (bLen>3) and (copy(sTmp,bLen-2,3)='ing') and
       (not isvowel((sTmp[bLen-3]))) then LikelyValid:=2;

    if bLen=4 then
      if (not IsVowel(sTmp[1])) and (IsVowel(sTmp[2]))
        and (not IsVowel(sTmp[3])) and (sTmp[4]='e') then LikelyValid:=1;

    (*Between here and "<<<end between here" based on lists taken from...
    "http://phrontistery.50megs.com/scrabble3.html

...The official list of acceptable words for American
       tournament play approved by the National Scrabble
       Association. It is based on the Official Scrabble
       Players' Dictionary (OSPD), with a few modifications
       to better reflect the rules of the game."

I didn't see any release of the author's copyright, so only a sample
to guide you has been left in this version of my program.

qTwo (<- search target specified in Dt3m.htm)
If you want to use the program to find words, as opposed to just as an example
of some Delphi programming, you'll need to get the list of words by a route
that reconciles your use and your conscience, and restore the rest of the lines
(in three places) along the pattern shown...*)
if bLen>3 then begin
(*qOther
The word to be checked is being checked in upper case just because
the list of vaild words was all in capitals, and I didn't want to
bother changing them.*)
sTmp:=uppercase(sTmp);
if (pos(sTmp,'AAH AAL AAS ABA ABO ABS ABY ACE ACT ADD ADO ADS ADZ AFF AFT AGA AGE AGO AHA AID')>0) then LikelyValid:=1;
if (pos(sTmp,'AIL AIM AIN AIR AIS AIT ALA ALB ALE ALL ALP ALS ALT AMA AMI AMP AMU ANA AND ANE')>0) then LikelyValid:=1;
(*.... etc... for the rest of the dictionary of real words*)
end; (*bLen>3*)


if bLen=2 then begin
LikelyValid:=5;
sTmp:=uppercase(sTmp);
if (pos(sTmp,'AA  AB  AD  AE  AG  AH  AI  AL  AM  AN  AR  AS  AT  AW  AX  AY  BA  BE  BI  BO')>0) then LikelyValid:=0;
(*.... etc... for the rest of the dictionary of real words*)
end; (*bLen=2*)
if bLen=3 then begin
LikelyValid:=5;
sTmp:=uppercase(sTmp);
if (pos(sTmp,'AAH AAL AAS ABA ABO ABS ABY ACE ACT ADD ADO ADS ADZ AFF AFT AGA AGE AGO AHA AID')>0) then LikelyValid:=0;
if (pos(sTmp,'AIL AIM AIN AIR AIS AIT ALA ALB ALE ALL ALP ALS ALT AMA AMI AMP AMU ANA AND ANE')>0) then LikelyValid:=0;
(*.... etc... for the rest of the dictionary of real words*)
    (*"<<<end between here", i.e. end of internet sourced material. See above.*)
end;

//Apply "If there's a q, it must be followed by u" test.
//Anyone know any other English words breaking the rule?
if (pos('q',sTmp)>0) and (pos('qu',sTmp)=0)
  and (sTmp<>'iraq') then LikelyValid:=3;

if bLen=1 then LikelyValid:=5;
if (sTmp='a') or (sTmp='i') then LikelyValid:=0;

end;(*has a vowel*)
end;(*function LikelyValid*)

begin (*LogResult*)
(*qMemoScroll (<- referred to in Dt3m.htm
The following two lines make the memo display results in a FIFO queue*)
me1.lines.delete(0);
me1.lines.add(sWord);
case LikelyValid(sWord) of
0:write(dfOut0,sWord+', ');
1:write(dfOut1,sWord+', ');
2:write(dfOut2,sWord+', ');
3:write(dfOut3,sWord+', ');
4:write(dfOut4,sWord+', ');
5:write(dfOut5,sWord+', ');
end;(*case*)
end;(*LogResult*)

procedure TDD43f1.eStartWordChange(Sender: TObject);
(*The reasons for this code are discussed in the tutorial at...
http://ourworld.compuserve.com/homepages/TK_Boyd/Dt3m.htm*)
begin
if (length(eStartWord.text)>2) and
   (length(eStartWord.text)<=maxLets) and
   (pos(' ',eStartWord.text)=0) then
   buGo.enabled:=true // no ; here
   else buGo.enabled:=false;
end;

procedure TDD43f1.FormCreate(Sender: TObject);
begin
DD43f1.caption:='DD43 Word Search, version: '+ver;
if fileexists('DD43R0.txt')then
  showmessage('A pre-exsiting DD43R0.txt is about to be overwritten.');

(*qOther This block is very crudely coded. Much could be done
to improve it, not least giving the user a way to terminate
execution of the program withOUT overwriting the pre-existing
datafiles!.... but this program is merely an example of how
to program for some OTHER functions, so time wasn't spent here*)
assignfile(dfOut0,'DD43R0.txt');
rewrite(dfOut0);
if fileexists('DD43R1.txt')then
  showmessage('A pre-exsiting DD43R1.txt is about to be overwritten.');
assignfile(dfOut1,'DD43R1.txt');
rewrite(dfOut1);
if fileexists('DD43R2.txt')then
  showmessage('A pre-exsiting DD43R2.txt is about to be overwritten.');
assignfile(dfOut2,'DD43R2.txt');
rewrite(dfOut2);
if fileexists('DD43R3.txt')then
  showmessage('A pre-exsiting DD43R3.txt is about to be overwritten.');
assignfile(dfOut3,'DD43R3.txt');
rewrite(dfOut3);
if fileexists('DD43R4.txt')then
  showmessage('A pre-exsiting DD43R4.txt is about to be overwritten.');
assignfile(dfOut4,'DD43R4.txt');
rewrite(dfOut4);
if fileexists('DD43R5.txt')then
  showmessage('A pre-exsiting DD43R5.txt is about to be overwritten.');
assignfile(dfOut5,'DD43R5.txt');
rewrite(dfOut5);
writeln(dfOut5,'If a word of less than 4 letters is not in the program''s');
writeln(dfOut5,'hard-coded dictionary, it will be placed in this output file.');
writeln(dfOut5,'- - - -');
end;

procedure TDD43f1.buGoClick(Sender: TObject);
var c1,c2,c3,c4,c5,c6,c7,c8:byte;
  bTmp,bLen:byte;
  sStartWord,sCombo:string;
  boUsed:array[0..maxLets] of boolean;

procedure ClearBoUsed;(*A sub-procedure of buGoClick*)
var c1:byte;
begin
for c1:=1 to maxLets do boUsed[c1]:=false;
end;

begin (*buGoClick*)
sStartWord:=lowercase(eStartWord.text);
(*qOther
Here's a loose end....
If you wanted to make this program better, here you would
sort the letters in sStartWord into alphabetical order so
that the output would be in alphabetical order... but
that is NOT done in the program as it stands. If you
want the words hidden in, say, "planets" to be generated,
in alphabetical order, just put "aelnpst" in the edit box.*)

(*qOther
  The following longish block of code (down to the first "closefile")
  is the "engine" that drives the word generation process. It has a few
  subtleties in it... the pattern is not quite as simple as it seems
  because it must handle exceptional circumstances at the inside (and at
  the outside?) parts of the nested loops. No rocket science here, but
  attention must be paid to detail. If you want an "exercise for the
  student", try extending the program to allow the permutations of
  8 and 9 letter words to be generated. It might be worth "turning off"
  the "write to file" and the "check if English" parts of the program
  while you get the loop structure working. Note that because of the
  modular design of the program, turning those things off is not
  hard. As another exercise: Try incorporating constants which allow
  you to turn either or both on/off just by changing the constants'
  values. If you enter, say, abcdefghi into the program, and the last
  "word" reported in the memo is ihgfedcba, (original, reversed), then
  you've probably got the extended looping structure right.

  Enjoy!
  *)
bLen:=length(sStartWord);
for c1:=1 to bLen do begin
ClearBoUsed;
  sCombo:=sStartWord[c1];
  LogResult(sCombo);
  boUsed[c1]:=true;
(*Next is "inside" previous, despite lack of indent*)
for c2:=1 to bLen do begin
  if boUsed[c2]=false then begin
    sCombo:=sCombo+sStartWord[c2];
    LogResult(sCombo);
    boUsed[c2]:=true;
(*Next is "inside" previous, despite lack of indent*)
for c3:=1 to bLen do begin
  if boUsed[c3]=false then begin
    sCombo:=sCombo+sStartWord[c3];
    LogResult(sCombo);
    boUsed[c3]:=true;
(*Next is "inside" previous, despite lack of indent*)
if bLen>3 then begin
for c4:=1 to bLen do begin
  if boUsed[c4]=false then begin
    sCombo:=sCombo+sStartWord[c4];
    LogResult(sCombo);
    boUsed[c4]:=true;
(*Next is "inside" previous, despite lack of indent*)
if bLen>4 then begin
for c5:=1 to bLen do begin
  if boUsed[c5]=false then begin
    sCombo:=sCombo+sStartWord[c5];
    LogResult(sCombo);
    boUsed[c5]:=true;
(*Next is "inside" previous, despite lack of indent*)
if bLen>5 then begin
for c6:=1 to bLen do begin
  if boUsed[c6]=false then begin
    sCombo:=sCombo+sStartWord[c6];
    LogResult(sCombo);
    boUsed[c6]:=true;
{>--}
(*Next is "inside" previous, despite lack of indent*)
if bLen>6 then begin
for c7:=1 to bLen do begin
  if boUsed[c7]=false then begin
    (*qOther: You should always have one of these inside
    loops that may tie up the machine.*)
    application.processmessages;
    sCombo:=sCombo+sStartWord[c7];
    LogResult(sCombo);
    boUsed[c7]:=true;
    (*Insert deeper layers HERE*)
    end;(*if boUsed[c7]...*)
end;(*for c7... begin..*)
end;(*if bLen>6...*)
sCombo:=copy(sCombo,1,5);
ClearBoUsed;
boUsed[c1]:=true;
boUsed[c2]:=true;
boUsed[c3]:=true;
boUsed[c4]:=true;
boUsed[c5]:=true;
{--}
    end;(*if boUsed[c6]...*)
end;(*for c6... begin..*)
end;(*if bLen>5...*)
sCombo:=copy(sCombo,1,4);
ClearBoUsed;
boUsed[c1]:=true;
boUsed[c2]:=true;
boUsed[c3]:=true;
boUsed[c4]:=true;
    end;(*if boUsed[c5]...*)
end;(*for c5... begin..*)
end;(*if bLen>4...*)
sCombo:=copy(sCombo,1,3);
ClearBoUsed;
boUsed[c1]:=true;
boUsed[c2]:=true;
boUsed[c3]:=true;
    end;(*if boUsed[c4]...*)
end;(*for c4... begin..*)
end;(*if bLen>3...*)
sCombo:=copy(sCombo,1,2);
ClearBoUsed;
boUsed[c1]:=true;
boUsed[c2]:=true;
    end;(*if boUsed[c3]...*)
end;(*for c3... begin..*)
sCombo:=sStartWord[c1];
ClearBoUsed;
boUsed[c1]:=true;
end;(*if boUsed[c2]=......*)
end;(*for c2... begin..*)
end;(*for c1... begin..*)

closefile(dfOut0);
closefile(dfOut1);
closefile(dfOut2);
closefile(dfOut3);
closefile(dfOut4);
closefile(dfOut5);
end;//buGoClick

end.
